#include <xparameters.h>
#include <xio.h>
#include "lcd.h"


//*****************************************************************************
//* Definitions.                                                              *
//*****************************************************************************
//LCD peripheral base address.
#define LCD_BASEADDR        XPAR_LCD_0_BASEADDR
//usec delay timer during initialization, important to change if clock speed changes.
#define INIT_DELAY          1000
//usec delay timer between instructions.
#define INST_DELAY          500
//usec delay timer between data.
#define DATA_DELAY          250


//*****************************************************************************
//* Internal functions.                                                       *
//*****************************************************************************
void usleep(unsigned long delay)
{
    volatile unsigned long i;
    volatile unsigned long j; 
    
    for (i = 0; i < delay; i++)
        for (j = 0; j < 26; j++) ;
}


void XromInitInst(void)
{
    XIo_Out32(LCD_BASEADDR, 0x00000003);
    usleep(1);
    XIo_Out32(LCD_BASEADDR, 0x00000043); //set enable and data
    usleep(1);
    XIo_Out32(LCD_BASEADDR, 0x00000003);
    usleep(INIT_DELAY);
}


void XromWriteInst(unsigned long inst1, unsigned long inst2)
{
    unsigned long printinst;

    printinst = 0x00000040 | inst1;

    XIo_Out32(LCD_BASEADDR, inst1);     //write data
    usleep(1);
    XIo_Out32(LCD_BASEADDR, printinst); //set enable
    usleep(1);
    XIo_Out32(LCD_BASEADDR, inst1);     //turn off enable
    usleep(1);

    printinst = 0x00000040 | inst2;

    XIo_Out32(LCD_BASEADDR, printinst); //set enable and data
    usleep(1);
    XIo_Out32(LCD_BASEADDR, inst2);     //turn off enable

    usleep(INST_DELAY);	
}


void XromWriteData(unsigned long data1, unsigned long data2)
{
    unsigned long rs_data;
    unsigned long enable_rs_data;	

    rs_data = (0x00000020 | data1);          //sets rs, data1
    enable_rs_data = (0x00000060 | data1);

    XIo_Out32(LCD_BASEADDR, rs_data);        //write data, rs
    usleep(1);
    XIo_Out32(LCD_BASEADDR, enable_rs_data); //set enable, keep data, rs
    usleep(1);
    XIo_Out32(LCD_BASEADDR, rs_data);        //turn off enable
    usleep(1);

    rs_data = (0x00000020 | data2);          //sets rs, data2
    enable_rs_data = (0x00000060 | data2);   //sets rs, data2

    XIo_Out32(LCD_BASEADDR, enable_rs_data); //set enable, rs, data
    usleep(1);
    XIo_Out32(LCD_BASEADDR, rs_data);        //turn off enable

    usleep(DATA_DELAY);
}


//*****************************************************************************
//* Moves the cursor home.                                                    *
//*****************************************************************************
void LCDMoveCursorHome()
{
    XromWriteInst(0x00000000, 0x00000002);
}


//*****************************************************************************
//* Moves the cursor left.                                                    *
//*****************************************************************************
void LCDMoveCursorLeft()
{
    XromWriteInst(0x00000001, 0x00000000);
}


//*****************************************************************************
//* Moves the cursor right.                                                   *
//*****************************************************************************
void LCDMoveCursorRight()
{
	XromWriteInst(0x00000001, 0x00000004);
}


//*****************************************************************************
//* Switches on the LCD display.                                              *
//*****************************************************************************
void LCDOn(unsigned long cursor_on, unsigned long cursor_blink)
{
	unsigned long command = 0x0C;
	
	if (cursor_on)
		command |= 0x02;
	if (cursor_blink)
		command |= 0x01;
	
    XromWriteInst(0x00000000, command);
}


//*****************************************************************************
//* Switches off the LCD display.                                             *
//*****************************************************************************
void LCDOff()
{
    XromWriteInst(0x00000000, 0x00000008);
}


//*****************************************************************************
//* Clears the LCD display.                                                   *
//*****************************************************************************
void LCDClear()
{
    XromWriteInst(0x00000000, 0x00000001);
    XromWriteInst(0x00000000, 0x00000010);
    LCDMoveCursorHome();
}


//*****************************************************************************
//* Initializes the LCD display.                                              *
//*****************************************************************************
void LCDInit()
{
    //Zeroes CHAR LCD Reg.
    XIo_Out32(LCD_BASEADDR, 0x00000000);

    //LCD INIT
    usleep(15000);	//After VCC>4.5V Wait 15ms to Init Char LCD
    XromInitInst();
    usleep(4100);    //Wait 4.1ms
    XromInitInst();
    usleep(100);     //Wait 100us
    XromInitInst();
    XromInitInst();

    //Function Set
    XromWriteInst(0x00000002, 0x00000008);

    //Entry Mode Set
    XromWriteInst(0x00000000, 0x00000006);	

    //Display Off
    XromWriteInst(0x00000000, 0x00000008);

    //Display On
    XromWriteInst(0x00000000, 0x0000000F);

    //Display Clear
    XromWriteInst(0x00000000, 0x00000001);
}


//*****************************************************************************
//* Sets the current line on the LCD display (line1 = 1, line2 = 2).          *
//*****************************************************************************
void LCDSetLine(unsigned long line)
{
    unsigned long i;

    if((line - 1) != 0)
    {
        LCDMoveCursorHome();
        for(i = 0; i < 40; i++)
            LCDMoveCursorRight();
    }
    else
    {
        LCDMoveCursorHome();
    }
}


//*****************************************************************************
//* Prints a character on the LCD display.                                    *
//*****************************************************************************
void LCDPrintChar(char ch)
{
    XromWriteData(((ch >> 4) & 0x0000000F), (ch & 0x0000000F));
}


//*****************************************************************************
//* Prints a null terminated string on the LCD display.                       *
//*****************************************************************************
void LCDPrintString(char *str)
{
    while(*str != 0)
    {
        LCDPrintChar(*str);
        str++;
    }
}


//*****************************************************************************
//* Prints two null terminated strings on the LCD display.                    *
//*****************************************************************************
void LCDPrint2Strings(char *str1, char *str2)
{
    unsigned long i = 0;
    
    LCDSetLine(1);

    for(i = 0; i < 16; i++)
    {
        if(str1[i] != 0)
	        LCDPrintChar(str1[i]);
        else
            break;
    }	

    LCDSetLine(2);

    for(i = 0; i < 16; i++)
    {
        if(str2[i] != 0)
	        LCDPrintChar(str2[i]);
        else
            break;
    }
}


//*****************************************************************************
//* Prints an unsigned number on the LCD display.                             *
//*****************************************************************************
void LCDPrintNum(unsigned long x, unsigned long base)
{
    static char hex[] = "0123456789ABCDEF";
    char digit[10];
    long i = 0;

    do
    {
        digit[i] = hex[x % base];
        x = x / base;
        i++;
    } while (x != 0);

    while (i > 0)
    {
        i--;
        LCDPrintChar(digit[i]);
    }
}


//*****************************************************************************
//* Prints a signed decimal number on the LCD display.                        *
//*****************************************************************************
void LCDPrintInt(long x)
{
    unsigned int val;

    if (x < 0)
    {
        LCDPrintChar('-');
        val = ((unsigned long) ~x) + 1;
    }
    else
    {
        val = (unsigned long) x;
    }

    LCDPrintNum(val, 10);
}
